@@ -28,12 +28,17 @@
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <libgnomevfs/gnome-vfs-directory.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include "file-utils.h"
 #include "fr-stock.h"
 #include "window.h"
 #include "gtk-utils.h"
 
 
+#define GLADE_FILE "file_roller.glade"
+
+
 typedef struct {
 	FRWindow  *window;
 	GtkWidget *dialog;
@@ -244,8 +249,8 @@ include_subfold_toggled_cb (GtkWidget *widget,
 }
 
 
-static void load_options_cb (GtkWidget *w, GtkWidget *data);
-static void save_options_cb (GtkWidget *w, GtkWidget *data);
+static void load_options_cb (GtkWidget *w, DialogData *data);
+static void save_options_cb (GtkWidget *w, DialogData *data);
 
 
 /* create the "add" dialog. */
@@ -404,12 +409,12 @@ add_cb (GtkWidget *widget,
 	g_signal_connect (G_OBJECT (data->load_button),
 			  "clicked",
 			  G_CALLBACK (load_options_cb), 
-			  file_sel);
+			  data);
 	
 	g_signal_connect (G_OBJECT (data->save_button),
 			  "clicked",
 			  G_CALLBACK (save_options_cb), 
-			  file_sel);
+			  data);
 	
 	g_object_set_data (G_OBJECT (file_sel), "tooltips", tooltips);
 
@@ -418,6 +423,27 @@ add_cb (GtkWidget *widget,
 }
 
 
+
+
+
+typedef struct {
+	DialogData   *data;
+	GladeXML     *gui;
+	GtkWidget    *dialog;
+	GtkWidget    *aod_treeview;
+	GtkTreeModel *aod_model;
+} LoadOptionsDialogData;
+
+
+static void
+aod_destroy_cb (GtkWidget             *widget,
+		LoadOptionsDialogData *aod_data)
+{
+	g_object_unref (aod_data->gui);
+	g_free (aod_data);
+}
+
+
 static gboolean
 config_get_bool (const char *config_file,
 		 const char *option)
@@ -449,105 +475,232 @@ config_get_string (const char *config_file,
 
 
 static void
-load_options_response_cb (GtkWidget *w,
-			  gint response,
-			  GtkWidget *opt_sel)
+aod_apply_cb (GtkWidget *widget,
+	      gpointer   callback_data)
 {
-	GtkWidget  *file_sel;
-	DialogData *data;
-	FRWindow   *window;
-	char       *opt_file_path;
-	char       *file_sel_path, *path;
-	gboolean    update;
-	gchar      *base_dir;
-	gboolean    recursive;
-	gboolean    no_symlinks;
-	gboolean    same_fs;
-	char       *exclude_files;
+	LoadOptionsDialogData *aod_data = callback_data;
+	DialogData            *data = aod_data->data;
+	GtkTreeSelection      *selection;
+	GtkTreeIter            iter;
+	char                  *file_path;
+	char                  *include_files = NULL;
+	char                  *exclude_files = NULL;
+	gboolean               update;
+	char                  *base_dir = NULL;
+	char                  *filename = NULL;
+	gboolean               recursive;
+	gboolean               no_symlinks;
 
-	if (response == GTK_RESPONSE_CANCEL) {
-		gtk_widget_destroy (opt_sel);
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (aod_data->aod_treeview));
+	if (! gtk_tree_selection_get_selected (selection, NULL, &iter)) 
 		return;
-	}
-	
-	file_sel = g_object_get_data (G_OBJECT (opt_sel), "fr_file_sel");
-	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
-	window = data->window;
 
-	opt_file_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (opt_sel));
-	
-        if (opt_file_path == NULL) {
-		g_free (opt_file_path);
-		gtk_widget_destroy (opt_sel);
-                return;
-	}
-	
+	gtk_tree_model_get (aod_data->aod_model, &iter,
+			    1, &file_path,
+			    -1);
+
 	/* Load options. */
 	
-	base_dir = config_get_string (opt_file_path, "base_dir");
-	path = config_get_string (opt_file_path, "include_files");
-	exclude_files = config_get_string (opt_file_path, "exclude_files");
-	update = config_get_bool (opt_file_path, "update");
-	recursive = config_get_bool (opt_file_path, "recursive");
-	no_symlinks = config_get_bool (opt_file_path, "no_symlinks");
-	same_fs = config_get_bool (opt_file_path, "same_fs");
+	base_dir = config_get_string (file_path, "base_dir");
+	filename = config_get_string (file_path, "filename");
+	include_files = config_get_string (file_path, "include_files");
+	exclude_files = config_get_string (file_path, "exclude_files");
+	update = config_get_bool (file_path, "update");
+	recursive = config_get_bool (file_path, "recursive");
+	no_symlinks = config_get_bool (file_path, "no_symlinks");
 
-	file_sel_path = g_strconcat (base_dir, "/", path, NULL);
+	g_free (file_path);
 
 	/* Sync widgets with options. */
 
+	if (base_dir != NULL)
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (data->dialog), base_dir);
+	/* FIXME
+	if (filename != NULL)
+		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (data->dialog), filename);
+	*/
+	if (include_files != NULL)
+		gtk_entry_set_text (GTK_ENTRY (data->include_files_entry), include_files);
+	if (exclude_files != NULL)
+		gtk_entry_set_text (GTK_ENTRY (data->exclude_files_entry), exclude_files);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->add_if_newer_checkbutton), update);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton), recursive);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_symlinks), no_symlinks);
-	gtk_entry_set_text (GTK_ENTRY (data->exclude_files_entry), exclude_files);
-	
-	/**/
 	
 	g_free (base_dir);
-	g_free (path);
-	g_free (file_sel_path);
+	g_free (filename);
+	g_free (include_files);
 	g_free (exclude_files);
-	g_free (opt_file_path);
+
+	/**/
+
+	gtk_widget_destroy (aod_data->dialog);
+}
+
+
+static void
+aod_activated_cb (GtkTreeView       *tree_view,
+                  GtkTreePath       *path,
+                  GtkTreeViewColumn *column,
+                  gpointer           callback_data)
+{
+	aod_apply_cb (NULL, callback_data);
+}
+
+
+static void
+aod_update_option_list (LoadOptionsDialogData *aod_data)
+{
+	GtkListStore   *list_store = GTK_LIST_STORE (aod_data->aod_model);
+	char           *options_dir;
+	GnomeVFSResult  result;
+	GList          *list = NULL;
 	
-	gtk_widget_destroy (opt_sel);
+	gtk_list_store_clear (list_store);
+
+	options_dir = get_home_relative_dir (RC_OPTIONS_DIR);
+	ensure_dir_exists (options_dir, 0700);
+	
+	result = gnome_vfs_directory_list_load (&list, 
+						options_dir,
+						GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+
+	if (result == GNOME_VFS_OK) {
+		GList *scan;
+
+		for (scan = list; scan; scan = scan->next) {
+			GnomeVFSFileInfo *file_info = scan->data;
+			GtkTreeIter       iter;
+			char             *full_path;
+
+			if ((strcmp (file_info->name, ".") == 0)
+			    || (strcmp (file_info->name, "..") == 0))
+				continue;
+			
+			full_path = g_build_filename (options_dir,
+						      file_info->name,
+						      NULL);
+			gtk_list_store_append (GTK_LIST_STORE (aod_data->aod_model),
+					       &iter);
+			gtk_list_store_set (GTK_LIST_STORE (aod_data->aod_model), &iter,
+					    0, file_info->name,
+					    1, full_path,
+					    -1);
+			g_free (full_path);
+		}
+	}
+	
+	gnome_vfs_file_info_list_free (list);
+	g_free (options_dir);
+}
+
+
+static void
+aod_remove_cb (GtkWidget             *widget,
+	       LoadOptionsDialogData *aod_data)
+{
+	GtkTreeSelection *selection;
+	GtkTreeIter       iter;
+	char             *file_path;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (aod_data->aod_treeview));
+	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
+		return;
+
+	gtk_tree_model_get (aod_data->aod_model, &iter,
+			    1, &file_path,
+			    -1);
+	gtk_list_store_remove (GTK_LIST_STORE (aod_data->aod_model), &iter);
+
+	gnome_vfs_unlink (file_path);
+	g_free (file_path);
 }
 
 
 static void 
 load_options_cb (GtkWidget  *w,
-		 GtkWidget  *file_sel)
+		 DialogData *data)
 {
-	GtkWidget  *opt_sel;
-	char       *options_dir;
-	char       *options_dir_slash;
-	
-	options_dir = get_home_relative_dir (RC_OPTIONS_DIR);
-	options_dir_slash = g_strconcat (options_dir, "/", NULL);
+	LoadOptionsDialogData *aod_data;
+	GtkWidget             *ok_button;
+	GtkWidget             *cancel_button;
+	GtkWidget             *remove_button;
+	GtkCellRenderer       *renderer;
+	GtkTreeViewColumn     *column;
 
-	ensure_dir_exists (options_dir, 0700);
-	
-	opt_sel = gtk_file_chooser_dialog_new (_("Load Options"),
-					       NULL,
-					       GTK_FILE_CHOOSER_ACTION_OPEN,
-					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-					       NULL);
-	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (opt_sel), 
-				       options_dir_slash);
+	aod_data = g_new0 (LoadOptionsDialogData, 1);
 
-	g_free (options_dir);
-	g_free (options_dir_slash);
+	aod_data->data = data;
+	aod_data->gui = glade_xml_new (GLADEDIR "/" GLADE_FILE , NULL, NULL);
+        if (! aod_data->gui) {
+                g_warning ("Could not find " GLADE_FILE "\n");
+		g_free (aod_data);
+                return;
+        }
+
+	/* Get the widgets. */
+
+	aod_data->dialog = glade_xml_get_widget (aod_data->gui, "add_options_dialog");
+	aod_data->aod_treeview = glade_xml_get_widget (aod_data->gui, "aod_treeview");
+
+	ok_button = glade_xml_get_widget (aod_data->gui, "aod_okbutton");
+	cancel_button = glade_xml_get_widget (aod_data->gui, "aod_cancelbutton");
+	remove_button = glade_xml_get_widget (aod_data->gui, "aod_remove_button");
+
+	/* Set the signals handlers. */
+
+	g_signal_connect (G_OBJECT (aod_data->dialog), 
+			  "destroy",
+			  G_CALLBACK (aod_destroy_cb),
+			  aod_data);
+	g_signal_connect (G_OBJECT (aod_data->aod_treeview),
+                          "row_activated",
+                          G_CALLBACK (aod_activated_cb),
+                          aod_data);
+	g_signal_connect_swapped (G_OBJECT (cancel_button), 
+				  "clicked",
+				  G_CALLBACK (gtk_widget_destroy),
+				  G_OBJECT (aod_data->dialog));
+	g_signal_connect (G_OBJECT (ok_button), 
+			  "clicked",
+			  G_CALLBACK (aod_apply_cb),
+			  aod_data);
+	g_signal_connect (G_OBJECT (remove_button), 
+			  "clicked",
+			  G_CALLBACK (aod_remove_cb),
+			  aod_data);
+
+	/* Set data. */
+
+	aod_data->aod_model = GTK_TREE_MODEL (gtk_list_store_new (2, 
+								  G_TYPE_STRING,
+								  G_TYPE_STRING));
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (aod_data->aod_model),
+                                              0,
+					      GTK_SORT_ASCENDING);
+	gtk_tree_view_set_model (GTK_TREE_VIEW (aod_data->aod_treeview),
+				 aod_data->aod_model);
+	g_object_unref (aod_data->aod_model);
+
+	/**/
+
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (NULL,
+							   renderer,
+							   "text", 0,
+							   NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 0);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (aod_data->aod_treeview),
+				     column);
 	
-	g_object_set_data (G_OBJECT (opt_sel), "fr_file_sel", file_sel);
-	
-	/* Signals */
-	g_signal_connect (G_OBJECT (opt_sel),
-			  "response",
-			  G_CALLBACK (load_options_response_cb), 
-			  opt_sel);
-	
-	gtk_window_set_modal (GTK_WINDOW (opt_sel), TRUE);
-	gtk_widget_show (opt_sel);
+	aod_update_option_list (aod_data);
+
+	/* Run */
+
+	gtk_window_set_transient_for (GTK_WINDOW (aod_data->dialog), 
+				      GTK_WINDOW (data->dialog));
+	gtk_window_set_modal (GTK_WINDOW (aod_data->dialog), TRUE);
+	gtk_widget_show (aod_data->dialog);
 }
 
 
@@ -582,52 +735,51 @@ config_set_string (const char *config_file,
 }
 
 
-static void
-save_options_response_cb (GtkWidget *w,
-			  gint response,
-			  GtkWidget *opt_sel)
+static void 
+save_options_cb (GtkWidget  *w,
+		 DialogData *data)
 {
-	DialogData *data;
-	FRWindow   *window;
-	GtkWidget  *file_sel;
+	char       *options_dir;
+	char       *opt_filename;
 	char       *opt_file_path;
-	char       *file_sel_path;
-	char       *path;
+	char       *base_dir;
+	char       *filename;
 	gboolean    update;
-	gchar      *base_dir;
 	gboolean    recursive;
 	gboolean    no_symlinks;
-	gboolean    same_fs;
+	const char *include_files;
 	const char *exclude_files;
 
-	if (response == GTK_RESPONSE_CANCEL) {
-		gtk_widget_destroy (opt_sel);
+	options_dir = get_home_relative_dir (RC_OPTIONS_DIR);
+	ensure_dir_exists (options_dir, 0700);
+
+	opt_filename = _gtk_request_dialog_run (GTK_WINDOW (data->dialog),
+						GTK_DIALOG_MODAL,
+						_("Save Options"),
+						_("Options Name:"),
+						"",
+						1024,
+						GTK_STOCK_CANCEL,
+						GTK_STOCK_SAVE);
+
+	if (opt_filename == NULL) 
 		return;
-	}
 
-	file_sel = g_object_get_data (G_OBJECT (opt_sel), "fr_file_sel");
-	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
-	window = data->window;
-	
-	opt_file_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (opt_sel));
+	opt_file_path = g_build_filename (options_dir, opt_filename, NULL);
+	g_free (opt_filename);
 
-        if (opt_file_path == NULL) {
-		g_free (opt_file_path);
-		gtk_widget_destroy (opt_sel);
-                return;
-	}
-	
 	/* Get options. */
 	
-	file_sel_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel));
-	path = remove_ending_separator (file_sel_path);
-	base_dir = remove_level_from_path (path);
-	
+	base_dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (data->dialog));
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->dialog));
 	update = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->add_if_newer_checkbutton));
-	
 	recursive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton));
 	no_symlinks = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_symlinks));
 
+	include_files = gtk_entry_get_text (GTK_ENTRY (data->include_files_entry));
+	if (utf8_only_spaces (include_files))
+		include_files = NULL;
+
 	exclude_files = gtk_entry_get_text (GTK_ENTRY (data->exclude_files_entry));
 	if (utf8_only_spaces (exclude_files))
 		exclude_files = NULL;
@@ -635,55 +787,17 @@ save_options_response_cb (GtkWidget *w,
 	/* Save options. */
 	
 	config_set_string (opt_file_path, "base_dir", base_dir);
-	config_set_string (opt_file_path, "include_files", file_name_from_path (path));
+	config_set_string (opt_file_path, "filename", filename);
+	config_set_string (opt_file_path, "include_files", include_files);
 	config_set_string (opt_file_path, "exclude_files", exclude_files);
 	config_set_bool   (opt_file_path, "update", update);
 	config_set_bool   (opt_file_path, "recursive", recursive);
 	config_set_bool   (opt_file_path, "no_symlinks", no_symlinks);
-	config_set_bool   (opt_file_path, "same_fs", same_fs);
 	gnome_config_sync ();
 
 	/**/
 	
-	g_free (path);
 	g_free (base_dir);
+	g_free (filename);
 	g_free (opt_file_path);
-	g_free (file_sel_path);
-	
-	gtk_widget_destroy (opt_sel);
-}
-
-
-static void 
-save_options_cb (GtkWidget  *w,
-		 GtkWidget  *file_sel)
-{
-	GtkWidget *opt_sel;
-	char      *options_dir;
-
-	options_dir = get_home_relative_dir (RC_OPTIONS_DIR);
-
-	ensure_dir_exists (options_dir, 0700);
-	
-	opt_sel = gtk_file_chooser_dialog_new (_("Save Options"),
-					       NULL,
-					       GTK_FILE_CHOOSER_ACTION_SAVE,
-					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					       GTK_STOCK_SAVE, GTK_RESPONSE_OK,
-					       NULL);
- 
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (opt_sel), options_dir);
-
-	g_free (options_dir);
-	
-	g_object_set_data (G_OBJECT (opt_sel), "fr_file_sel", file_sel);
-	
-	/* Signals */
-	g_signal_connect (G_OBJECT (opt_sel),
-			  "response",
-			  G_CALLBACK (save_options_response_cb), 
-			  opt_sel);
-	
-	gtk_window_set_modal (GTK_WINDOW (opt_sel), TRUE);
-	gtk_widget_show (opt_sel);
 }
