@@ -20,6 +20,7 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -435,57 +436,50 @@ path_list_dup (GList *path_list)
 }
 
 
-/* characters to escape */
-static gchar bad_char[] = { '$', '\'', '`', '"', '\\', '!', '?', '*',
-			    ' ', '(', ')', '[', ']', '&', '|', '@', '#',
-			    ':', ';', '=', '-' };
-
-/* the size of bad_char */
-static const gint bad_chars = sizeof (bad_char) / sizeof (gchar);
-
-
 /* counts how many characters to escape in @str. */
-static gint
-count_chars_to_escape (const gchar *str)
+static int
+count_chars_to_escape (const char *str, 
+		       const char *meta_chars)
 {
-	const gchar *s;
-	gint i, n;
+	int         meta_chars_n = strlen (meta_chars);
+	const char *s;
+	int         n = 0;
 
-	n = 0;
-	for (s = str; *s != 0; s++)
-		for (i = 0; i < bad_chars; i++)
-			if (*s == bad_char[i]) {
+	for (s = str; *s != 0; s++) {
+		int i;
+		for (i = 0; i < meta_chars_n; i++) 
+			if (*s == meta_chars[i]) {
 				n++;
 				break;
 			}
+	}
 	return n;
 }
 
 
-/* escape with backslash the file name. */
-gchar*
-shell_escape (const gchar *filename)
+/* escape with backslash the string @str. */
+char*
+escape_str (const char *str, 
+	    const char *meta_chars)
 {
-	gchar *escaped;
-	gint i, new_l;
-	const gchar *s;
-	gchar *t;
+	int         meta_chars_n = strlen (meta_chars);
+	char       *escaped;
+	int         i, new_l;
+	const char *s;
+	char       *t;
 
-	if (filename == NULL) 
+	if (str == NULL) 
 		return NULL;
 
-	new_l = strlen (filename) + count_chars_to_escape (filename);
+	new_l = strlen (str) + count_chars_to_escape (str, meta_chars);
 	escaped = g_malloc (new_l + 1);
 
-	s = filename;
+	s = str;
 	t = escaped;
 	while (*s) {
-		gboolean is_bad;
-	
-		is_bad = FALSE;
-		for (i = 0; (i < bad_chars) && !is_bad; i++)
-			is_bad = (*s == bad_char[i]);
-
+		gboolean is_bad = FALSE;
+		for (i = 0; (i < meta_chars_n) && !is_bad; i++)
+			is_bad = (*s == meta_chars[i]);
 		if (is_bad)
 			*t++ = '\\';
 		*t++ = *s++;
@@ -496,6 +490,14 @@ shell_escape (const gchar *filename)
 }
 
 
+/* escape with backslash the file name. */
+char*
+shell_escape (const char *filename)
+{
+	return escape_str (filename, "$\'`\"\\!?* ()[]&|@#:;=-");
+}
+
+
 static gchar *
 get_terminal ()
 {
@@ -680,7 +682,7 @@ g_utf8_strchug (char *string)
 {
 	char     *scan;
 	gunichar  c;
-                                                                                
+
 	g_return_val_if_fail (string != NULL, NULL);
 	
 	scan = string;
@@ -690,7 +692,7 @@ g_utf8_strchug (char *string)
 		c = g_utf8_get_char (scan);
 	}
 
-	g_memmove (string, scan, strlen ((char *) scan) + 1);
+	g_memmove (string, scan, strlen (scan) + 1);
 	
 	return string;
 }
@@ -699,13 +701,18 @@ g_utf8_strchug (char *string)
 static char*
 g_utf8_strchomp (char *string)
 {
-	char  *scan;
-	gsize  len;
+	char   *scan;
+	gsize   len;
  
 	g_return_val_if_fail (string != NULL, NULL);
 	
 	len = g_utf8_strlen (string, -1);
-	scan = g_utf8_offset_to_pointer (string, len);
+
+	if (len == 0)
+		return string;
+
+	scan = g_utf8_offset_to_pointer (string, len - 1);
+
 	while (len--) {
 		gunichar c = g_utf8_get_char (scan);
 		if (g_unichar_isspace (c))
@@ -719,7 +726,7 @@ g_utf8_strchomp (char *string)
 }
 
 
-#define g_utf8_strstrip( string )    g_utf8_strchomp (g_utf8_strchug (string))
+#define g_utf8_strstrip(string)    g_utf8_strchomp (g_utf8_strchug (string))
 
 
 char **
@@ -934,20 +941,20 @@ get_last_field (const char *line,
 
 
 char *
-get_temp_work_dir_name ()
+get_temp_work_dir_name (void)
 {
 	char       *result = NULL;
 	static int  count = 0;
 	int         try = 0;
 
 	do {
+		g_free (result);
 		result = g_strdup_printf ("%s%s.%d.%d",
 					  g_get_tmp_dir (),
 					  "/file-roller",
 					  getpid (),
 					  count++);
-	} while (g_file_test (result, G_FILE_TEST_EXISTS)
-		 && (try++ < MAX_TRIES));
+	} while (path_is_file (result) && (try++ < MAX_TRIES));
 
 	return result;
 }
